
import "WalmPlugins.YourOwnAdventures.StepReactorTARGET";
import "WalmPlugins.YourOwnAdventures.StepReactorCHATLOG";
import "WalmPlugins.YourOwnAdventures.StepReactorQUICKSLOTS";
import "WalmPlugins.YourOwnAdventures.StepReactorTALKTREE";
import "WalmPlugins.YourOwnAdventures.StepReactorKILL";
import "WalmPlugins.YourOwnAdventures.StepReactorITEMACTION";
import "WalmPlugins.YourOwnAdventures.StepReactorWAIT";
import "WalmPlugins.YourOwnAdventures.StepReactorTRADE";

import "WalmPlugins.YourOwnAdventures.DeedsEngine";

AdventureEngine = class();

function AdventureEngine:Constructor()
	self.quest = nil;
	self.reactors = {};
end

function AdventureEngine:LoadQuest(quest)
	if(type(quest)=="string") then
		quest = self:GetAdventureById(quest);
	end

	if(quest == nil) then
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>YourOwnAdventures: ".._("Q_LOAD_ERR").."</rgb>");
		return false;
	end

	if(PROGRESS.ADVENTURES[quest.uniqueID]~=nil and PROGRESS.ADVENTURES[quest.uniqueID].stepId=="END") then
		return false;
	end
	self:UnLoadQuest();
	self.quest = quest;
	PROGRESS.lastAdventure = quest.uniqueID;
	SaveProgress();

	if(self.quest.title==nil) then self.quest.title=TRAD("L'aventure sans nom", "The unamed adventure"); end
	quickLauncher.questName:SetText(FormatAdventureString(self.quest.title));
	Turbine.Shell.WriteLine("<rgb=#FF5CCD>".._("Q_LOAD_OK")..FormatAdventureString(self.quest.title).."</rgb>");	
	self:LoadQuestSteps();
	AdvDiary:UpdateDiayData(self.quest, "LOAD");
	return true;
end

function AdventureEngine:UnLoadQuest()
	self:ClearAllReactors(self.reactors);
	self.reactors = {};

	AdvDiary:UpdateDiayData(self.quest, "UNLOAD");
	self.quest = nil;
	PROGRESS.lastAdventure = nil;
	SaveProgress();
	quickLauncher.questName:SetText("");
	quickLauncher:RemoveObjectives();
	quickLauncher:StopAdventureTimer();
	campaignLoader.questPane:SetVisible(false);
	campaignLoader:UpdateTreeViewStatus();
end

function AdventureEngine:UnLoadFinishedQuest(stepId)
	if(self.quest~=nil) then
		if(stepId=="FAIL") then
			splashMessage:AddMessage(_("Q_FAILED")..self.quest.title);
			Turbine.Shell.WriteLine("<rgb=#FF0000>".._("Q_FAILED").."</rgb>"..FormatAdventureString(self.quest.title));
			if(PROGRESS.ADVENTURES[self.quest.uniqueID]==nil) then PROGRESS.ADVENTURES[self.quest.uniqueID]={}; end
			PROGRESS.ADVENTURES[self.quest.uniqueID] = {};
			PROGRESS.ADVENTURES[self.quest.uniqueID].stepId = "START";
			SaveProgress();
		else
			local firstCompletion = false;
			splashMessage:AddMessage(_("Q_FINISHED")..self.quest.title);
			deedsEngine:AddDeed(self.quest.uniqueID);
			Turbine.Shell.WriteLine("<rgb=#00FF00>".._("Q_FINISHED").."</rgb>"..FormatAdventureString(self.quest.title));
			if(PROGRESS.ADVENTURES[self.quest.uniqueID]==nil) then PROGRESS.ADVENTURES[self.quest.uniqueID]={}; end
			PROGRESS.ADVENTURES[self.quest.uniqueID].stepId = "END";
			if(PROGRESS.ACHIEVEMENTS[self.quest.uniqueID]==nil) then
				PROGRESS.ACHIEVEMENTS[self.quest.uniqueID]={};
			end
			PROGRESS.ADVENTURES[self.quest.uniqueID].timer = nil;
			if(PROGRESS.ACHIEVEMENTS[self.quest.uniqueID].firstCompletionDate==nil) then
				firstCompletion = true;
				PROGRESS.ACHIEVEMENTS[self.quest.uniqueID].firstCompletionDate = Turbine.Engine.GetLocalTime();
			end
			if(PROGRESS.ACHIEVEMENTS[self.quest.uniqueID].count==nil) then
				PROGRESS.ACHIEVEMENTS[self.quest.uniqueID].count=0;
				PROGRESS.XP.questFinished = PROGRESS.XP.questFinished + 1;
			end
			PROGRESS.ACHIEVEMENTS[self.quest.uniqueID].lastCompletionDate = Turbine.Engine.GetLocalTime();
			PROGRESS.ACHIEVEMENTS[self.quest.uniqueID].count = PROGRESS.ACHIEVEMENTS[self.quest.uniqueID].count + 1;
			SaveProgress();
			if(firstCompletion==true) then
				playersheet:OnAdventureUpdated(self.quest.family);
				deedsEngine:AddDeed({"GENERIC_DEEDS_A1", "GENERIC_DEEDS_A2", "GENERIC_DEEDS_A3", "GENERIC_DEEDS_A4"});		
			end
		end
		local last = self.quest;
		self:UnLoadQuest();
		quickLauncher:ShowSuggestedAdventures(last);
	else
		self:UnLoadQuest();
	end
end

function AdventureEngine:LoadQuestSteps()
	if(self.quest.steps == nil) then 
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>YourOwnAdventures: ".._("QS_LOAD_ERR").."</rgb>");
		return false;
	end
	
	-- vérifier la sauvegarde
	local stepId = "START";
	
	if(PROGRESS.ADVENTURES[self.quest.uniqueID]~=nil) then
		if(PROGRESS.ADVENTURES[self.quest.uniqueID].stepId ~= nil) then
			stepId = PROGRESS.ADVENTURES[self.quest.uniqueID].stepId;
		end
		if(PROGRESS.ADVENTURES[self.quest.uniqueID].timer ~= nil) then
			quickLauncher:InitAdventureTimer(PROGRESS.ADVENTURES[self.quest.uniqueID].timer);
		end		
	else	
		if(self.quest.loadingDiaryMessage ~= nil) then
			PROGRESS.ADVENTURES[self.quest.uniqueID] = {};
			PROGRESS.ADVENTURES[self.quest.uniqueID].diary = {};
			table.insert(PROGRESS.ADVENTURES[self.quest.uniqueID].diary, self.quest.loadingDiaryMessage);
			SaveProgress();			
		end
	end
	
	local result = self:LoadQuestStep(stepId);
	
	-- TODO savec LoadTimer
	return result;
end

function AdventureEngine:LoadQuestStep(stepId, isRecursive)
	
	if(type(stepId)=="table") then
		self.reactors = {};
		for i, id in pairs(stepId) do
			if(type(id)=="string") then
				local step = self:LoadQuestStep(id, true);
				if(step~=nil and step~=false) then
					table.insert(self.reactors, step);
					-- if(objective
					-- Turbine.Shell.WriteLine("<rgb=#FF5CCD>"..TRAD("Nouvel objectif : ", "New objective : ")..(objective or "NIHIL"));
					quickLauncher:AddObjective(step);
				end
			elseif(type(id)=="table") then
				local tab = {};
				for j, id2 in pairs(id) do
					if(type(id2)=="string") then
						local step = self:LoadQuestStep(id2, true);
						if(step~=nil and step~=false) then
							table.insert(tab, step);
						end
					end
				end
				if(table.getn(tab)>0) then
					table.insert(self.reactors, tab);
					quickLauncher:AddObjective(tab);
				end
			end
		end
	end

	local save = not isRecursive;
	if (self.quest~=nil) then
		if(self.quest.steps~=nil and type(stepId)=="string" and self.quest.steps[stepId]~=nil and self.quest.steps[stepId].saveProgress == false) then	
			save = false;
		end
		if(self.quest.steps[stepId]~=nil and self:TestSuccessCondition(self.quest.steps[stepId].loadingCondition)==false) then
			return false;
		end
		if(save==true and stepId~=nil and self.quest~=nil) then
			if(PROGRESS.ADVENTURES[self.quest.uniqueID]==nil) then PROGRESS.ADVENTURES[self.quest.uniqueID]={}; end
			
			local stepIdToSave = stepId;
			if((stepId~="END" or stepId~="FAIL") and type(stepId)=="string" and self.quest.steps[stepId]~=nil) then 
				if(self.quest.steps[stepId].stepIdToSave~=nil) then
					stepIdToSave = self.quest.steps[stepId].stepIdToSave; 
				end
			end
			PROGRESS.ADVENTURES[self.quest.uniqueID].stepId = stepIdToSave;
			
			SaveProgress();
		end
	end
	if(type(stepId)=="table") then
		return nil;
	end
	---------------------------------------------------------------------------
	-- if(stepId==nil) then
		-- if(table.getn(self.reactors)>1) then
			-- return true;
		-- else
			-- stepId="END";
		-- end
	-- end
	if(stepId=="END" or stepId=="FAIL") then
		self:UnLoadFinishedQuest(stepId);
		return nil;
	end
	
	if(self.quest.steps[stepId]==nil) then
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>YourOwnAdventures: ".._("QS_LOAD_UNKSTEPID")..stepId.."</rgb>");
		return nil;
	end
	
	local step = nil;
	local stepType = self.quest.steps[stepId].steptype;
	if(stepType == "TARGET") then
		step = StepReactorTARGET(self.quest, stepId);
	elseif(stepType == "CHATLOG") then
		step = StepReactorCHATLOG(self.quest, stepId);
	elseif(stepType == "KILL") then
		step = StepReactorKILL(self.quest, stepId);
	elseif(stepType == "QUICKSLOTS") then
		step = StepReactorQUICKSLOTS(self.quest, stepId);
	elseif(stepType == "ITEMACTION") then
		step = StepReactorITEMACTION(self.quest, stepId);	
	elseif(stepType == "TALKTREE") then
		step = StepReactorTALKTREE(self.quest, stepId);
	elseif(stepType == "TRADE") then
		step = StepReactorTRADE(self.quest, stepId);
	elseif(stepType == "KILL") then
		step = StepReactorKILL(self.quest, stepId);
	elseif(stepType == "WAIT") then
		step = StepReactorWAIT(self.quest, stepId);
	else
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>YourOwnAdventures: ".._("QS_LOAD_UNKSTETYPE")..stepType.."</rgb>");
		return nil;
	end

	if(isRecursive~=true) then
		self.reactors = {step};
		quickLauncher:AddObjective(step);
	end
	
	self:ApplyStepGenericActions(self.quest.steps[stepId].onLoading);
	
	--if(quickLauncher:GetAdventureTimer()==nil) then
		if(self.quest.steps[stepId].timer) then
			quickLauncher:InitAdventureTimer(self.quest.steps[stepId].timer, step);
		end
	--end
	
	return step;
end

function AdventureEngine:ItemAction(action, item, reactors)
	if(reactors==nil) then reactors=self.reactors; end
	for i, s in pairs(reactors) do
		if(s.ItemAction ~= nil) then
			s:ItemAction(action, item);
		elseif(s.stepId==nil and type(s)=="table") then
			self:ItemAction(action, item, s);
		end
	end	
end

function AdventureEngine:TestSuccessCondition(successCondition, sourceAdventure)
	if(successCondition==nil) then
		return true;
	end
	local success = true;
	if(successCondition.requireItem~=nil) then
		success = success and inventoryEngine:HasItem(successCondition.requireItem);
	end
	if(successCondition.requireVariable~=nil) then
		success = success and self:HasVariable(successCondition.requireVariable);
	end
	if(successCondition.requireAdventure~=nil) then
		if(type(successCondition.requireAdventure)=="string" and self.quest~=nil) then
			success = success and (successCondition.requireAdventure==self.quest.uniqueID);
		elseif(type(successCondition.requireAdventure)=="table" and self.quest~=nil) then
			local found = false;
			for i, str in pairs(successCondition.requireAdventure) do
				if(str==self.quest.uniqueID) then found=true; end
			end
			succes = succes and found;
		end
	end
	if(success==false and type(successCondition.ifConditionsFailed)=="table") then
		self:ApplyStepGenericActions(successCondition.ifConditionsFailed, sourceAdventure);
	end
	return success;
end


function AdventureEngine:ApplyStepGenericActions(step, sourceAdventure)
	if(step == nil) then return; end
	local adventure = sourceAdventure or self.quest;
	if(step.pushSystemChat ~= nil) then
		Turbine.Shell.WriteLine(FormatAdventureString(step.pushSystemChat));
	end
	if(step.pushSplashMessage ~= nil) then
		if(type(step.pushSplashMessage)=="string") then
			splashMessage:AddMessage(step.pushSplashMessage);
		elseif(type(step.pushSplashMessage)=="table") then
			for i, j in pairs(step.pushSplashMessage) do
				splashMessage:AddMessage(j);
			end		
		end
	end
	if(step.cancelTimer == true) then
		quickLauncher:StopAdventureTimer();
	end
	if(step.accomplishDeed ~= nil) then
		deedsEngine:AddDeed(step.accomplishDeed);
	end
	if(step.pushDiaryMessage ~= nil) then
		if(PROGRESS.ADVENTURES[adventure.uniqueID]==nil) then
			PROGRESS.ADVENTURES[adventure.uniqueID] = {};
		end
		if(PROGRESS.ADVENTURES[adventure.uniqueID].diary==nil) then
			PROGRESS.ADVENTURES[adventure.uniqueID].diary = {};
		end
		local found = false;
			for i, msg in pairs(PROGRESS.ADVENTURES[adventure.uniqueID].diary) do
				if(msg==step.pushDiaryMessage) then found=true; end
			end
		if(found==false) then
			local message = FormatAdventureString(step.pushDiaryMessage);
			table.insert(PROGRESS.ADVENTURES[adventure.uniqueID].diary, message);
			SaveProgress();	
			AdvDiary:UpdateDiayData(adventure, "UPDATE");
		end
	end
	if(step.pushDiaryPicture ~= nil) then
		if(PROGRESS.ADVENTURES[adventure.uniqueID]==nil) then
			PROGRESS.ADVENTURES[adventure.uniqueID] = {};
		end
		if(PROGRESS.ADVENTURES[adventure.uniqueID].diary==nil) then
			PROGRESS.ADVENTURES[adventure.uniqueID].diary = {};
		end
		table.insert(PROGRESS.ADVENTURES[adventure.uniqueID].diary, step.pushDiaryPicture);
		SaveProgress();			
	end
	if(step.setVariable ~= nil) then
		if(PROGRESS.VARIABLES==nil) then PROGRESS.VARIABLES = {}; end
		local array = step.setVariable;
		if(type(array[1])=="string") then
			array = {step.setVariable};
		end
		if(array~=nil and type(array)=="table") then
			for i, t in pairs(array) do

				if(type(t=="table") and type(t[1]=="string")) then			
					local value = 0;
					if(type(t[3])=="number") then value=t[3]; end
					if(t[4]~=nil and type(t[4])=="number") then value=math.random(value, t[4]); end
					if(PROGRESS.VARIABLES[t[1]]==nil) then PROGRESS.VARIABLES[t[1]]=0; end
					if(t[2]== "=") then
						PROGRESS.VARIABLES[t[1]] = value;
					elseif(t[2]== "+") then
						PROGRESS.VARIABLES[t[1]] = PROGRESS.VARIABLES[t[1]] + value;
					elseif(t[2]== "-") then
						PROGRESS.VARIABLES[t[1]] = PROGRESS.VARIABLES[t[1]] - value;
					end
					if(PROGRESS.VARIABLES[t[1]]==0) then PROGRESS.VARIABLES[t[1]]=nil; end
				end
			end
			SaveProgress();
		end
	end		
	if(step.removeItem ~= nil) then
		if(type(step.removeItem)=="string") then
			inventoryEngine:RemoveItem(step.removeItem, 1);
		elseif(type(step.removeItem)=="table") then
			for i, j in pairs(step.removeItem) do
				inventoryEngine:RemoveItem(j, 1);
			end		
		end
	end		
	if(step.giveItem ~= nil) then
		if(type(step.giveItem)=="string") then
			inventoryEngine:AddItem(step.giveItem, 1);
		elseif(type(step.giveItem)=="table") then
			for i, j in pairs(step.giveItem) do
				inventoryEngine:AddItem(j, 1);
			end		
		end
	end

end

function AdventureEngine:ClearAllReactors(reactorArray)
	if(reactorArray~=nil) then
		for i, s in pairs(reactorArray) do
			if(s.Unload~=nil) then 
				quickLauncher:RemoveObjective(s); -- ???
				s:Unload();
				s = nil;
			elseif(type(s)=="table") then
				self:ClearAllReactors(s);
			end
		end	
	end	
end

function AdventureEngine:ClearSynchronousReactors(step)
	if(step==nil or self.reactors==nil) then return; end
	
	for i, s in pairs(self.reactors) do
		if(s.stepId~=nil) then
			if(step==s) then
				quickLauncher:RemoveObjective(step);
				table.remove(self.reactors, i);
				step:Unload();
			end
		else
			for j, s2 in pairs(s) do
				if(s2.stepId~=nil) then
					if(step==s2) then
						if(step.step.nextStep==nil and table.getn(s)>1) then
							--
						else
							quickLauncher:RemoveObjective(step);
							self:ClearAllReactors(self.reactors[i]);
							table.remove(self.reactors[i], j);
							table.remove(self.reactors, i);
							--step:Unload();
						end
					end
				end
			end
		end
	end
	if(table.getn(self.reactors)==0) then
		quickLauncher:RemoveObjectives();
		if(step.step~=nil and self.quest~=nil) then
			PROGRESS.ADVENTURES[self.quest.uniqueID].timer = quickLauncher:GetAdventureTimer();
			self:LoadQuestStep(step.step.nextStep);
			SaveProgress();
		end
	else
		local saveArray = {};
		for i, s in pairs(self.reactors) do
			if(s.stepId~=nil) then
				table.insert(saveArray, s.stepId);
			else
				local array = {};
				for j, s2 in pairs(s) do
					if(s2.stepId~=nil) then
						table.insert(array, s2.stepId);
					end
				end
				if(table.getn(array)>0) then
					table.insert(saveArray, array);
				end
			end
		end		
		PROGRESS.ADVENTURES[self.quest.uniqueID].stepId = saveArray;	
		PROGRESS.ADVENTURES[self.quest.uniqueID].timer = quickLauncher:GetAdventureTimer();
		SaveProgress();
	end
end

function AdventureEngine:GetActiveQuest()
	return self.quest;
end

function AdventureEngine:BuildFamilyList()
	local familyList = {};
	for i, deed in pairs(DEEDS) do
		if(ListContainsString(familyList, deed.family)==false) then
			if(not (Preferences.Filters.tech==false and deed.tech==true)) then 
				table.insert(familyList, deed.family);
			end
		end
	end	
	local m = _("GENERIC_DEEDS");
	table.sort(familyList, function(a,b) if(a==m)then return true; end if(b==m)then return false; end return (a<b); end);
	return familyList;
end

function AdventureEngine:GetAdventureById(advId)
	for i=1, table.getn(ADVENTURES), 1 do
		local family = ADVENTURES[i].family or _("ORPHANS");
		if(ADVENTURES[i].uniqueID == advId) then
			return ADVENTURES[i];
		end
	end
	return nil;
end

function AdventureEngine:ShowSteps()
	if(self.quest==nil) then
		Turbine.Shell.WriteLine("<rgb=#FFFFFF>YourOwnAdventures: "..TRAD("Aucune aventure chargée...", "No running adventure...").."</rgb>");
	else
		Turbine.Shell.WriteLine("<rgb=#FFFFFF>YourOwnAdventures: "..TRAD("Aventure chargée : ", "Loaded adventure : ")..self.quest.title.."</rgb>\n");
		Turbine.Shell.WriteLine("<rgb=#FFFFFF>"..table.getn(self.reactors)..TRAD(" étapes en cours :", " running steps :"));
		for i, s in pairs(self.reactors) do
			if(s.stepId~=nil) then
				Turbine.Shell.WriteLine("<rgb=#FFFFFF>OBJECTIF #"..i..": id="..s.stepId.." steptype=["..s.step.steptype.."]");
			else
				Turbine.Shell.WriteLine("<rgb=#FFFFFF>OBJECTIF #"..i.." NODE :");
				for j, s2 in pairs(s) do
					if(s2.stepId~=nil) then
						Turbine.Shell.WriteLine("<rgb=#FFFFFF>=====> #"..j..": id="..s2.stepId.." steptype=["..s2.step.steptype.."]");
					end
				end
			end
		end
		-- Turbine.Shell.WriteLine("<rgb=#FFFFFF>YourOwnAdventures: "..TRAD("Aventure chargée : ", "Loaded adventure : ")..self.quest.title.."</rgb>\n");
		-- Turbine.Shell.WriteLine("<rgb=#FFFFFF>"..table.getn(self.reactors)..TRAD(" étapes en cours.", " running steps."));
		-- for i, s in pairs(self.reactors) do
			-- Turbine.Shell.WriteLine("<rgb=#FFFFFF>#"..i..": id="..s.stepId.." steptype=["..s.step.steptype.."]");
		-- end
	end
end

function AdventureEngine:HasVariable(variables)
	local array = variables;
	if(type(array)=="table" and type(array[1])=="string") then
		array = {array};
	end
	local result = true;
	if(array~=nil and type(array)=="table") then
		for i, t in pairs(array) do
			--Turbine.Shell.WriteLine("<rgb=#FFFFFF>YourOwnAdventures: #"..i);
			if(type(t=="table") and type(t[1]=="string")) then			
				local value = 0;
				if (type(t[3])=="number") then value=t[3]; end
				if(t[4]~=nil and type(t[4])=="number") then value=math.random(value, t[4]); end
				local mem = PROGRESS.VARIABLES[t[1]];
				if(mem==nil) then mem=0; end
				if(t[2]== "=") then
					result = result and (mem==value);
				elseif(t[2]== ">") then
					result = result and (mem>value);
				elseif(t[2]== "<") then
					result = result and (mem<value);
				elseif(t[2]== ">=") then
					result = result and (mem>=value);
				elseif(t[2]== "<=") then
					result = result and (mem<=value);
				end
			end
		end
	end
	return result;
end

function AdventureEngine:ShowVariables()
	Turbine.Shell.WriteLine("<rgb=#FF5CCD>Variables sauvegardées :");
	for i, s in pairs(PROGRESS.VARIABLES) do
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>* "..i.." value="..s);
	end
end

function AdventureEngine:IsCurrentAdventure(advId)
	if(type(advId)=="string" and self.quest~=nil) then
		return (advId==self.quest.uniqueID);
	elseif(type(advId)=="table") then
		for i, j in pairs(step.giveItem) do
			if(self:IsCurrentAdventure()==true) then
				return true;
			end
		end		
	end
	return false;
end

function AdventureEngine:HasFinished(adventureID)
	if(adventureID==nil) then return false; end
	if(type(adventureID)=="table") then
		local ok = true;
		for i, ID in pairs(adventureID) do
			ok = ok and self:HasFinished(ID);
		end
		return ok;
	elseif(type(adventureID)=="string") then
		return (PROGRESS.ACHIEVEMENTS[adventureID]~=nil);
	end
	return false;
end

function RAND(list)
	if(list==nil or type(list)~="table") then return nil; end
	local r = math.random(table.getn(list));
	return list[r];
end

function FormatAdventureString(str)
	if(str==nil) then return nil; end
	return string.gsub(str, "|PLAYER|", Turbine.Gameplay.LocalPlayer.GetInstance():GetName());
end