

CampaignSelectorPane = class( Turbine.UI.Label );

function CampaignSelectorPane:Constructor(width, height)	  	
	Turbine.UI.Label.Constructor( self );    	
	self:SetSize( width, height );
	self.quest = nil;
	
	self:CreateHeader();
	self:CreateDescription();
	self:CreateQuestButtons();
	self:SetVisible(false);
end



function CampaignSelectorPane:CreateQuestButtons()
	
	self.launchButton = Turbine.UI.Button();
	self.launchButton:SetParent(self);
	self.launchButton:SetBackground("WalmPlugins/YourOwnAdventures/pictures/campaigns/launchButton.tga");
	self.launchButton:SetPosition(5, 393);
	self.launchButton:SetSize(200, 60);	
	self.launchButton:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	self.launchButton:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16);
	self.launchButton:SetForeColor(Turbine.UI.Color( 1, 223/255, 0 ));
	self.launchButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.launchButton.MouseClick = function(sender, args)
		local result = AdventureEngine:LoadQuest(self.quest);
		if(result==true) then
			campaignLoader:SetVisible(false);
		end
	end	
	
	self.restartButton = Turbine.UI.Lotro.Button();
	self.restartButton:SetParent(self);
	self.restartButton:SetText(_("RESTART"));
	self.restartButton:SetPosition(205, 398);
	self.restartButton:SetSize(175, 24);
	
	self.shareButton = Turbine.UI.Lotro.Button();
	self.shareButton:SetParent(self);
	self.shareButton:SetText(_("SHARE"));
	self.shareButton:SetPosition(205, 422);
	self.shareButton:SetSize(175, 24);	
end

function CampaignSelectorPane:CreateDescription()
	local background = Turbine.UI.Label();
	background:SetParent( self );
	background:SetSize(370, 200);
	background:SetPosition((self:GetWidth()-370)/2, 120);	
	background:SetMouseVisible( false );
	background:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	background:SetBackground("WalmPlugins/YourOwnAdventures/pictures/campaigns/parchment.jpg");
	
	self.descriptionArea = Turbine.UI.Label();
	self.descriptionArea:SetParent( background );
	self.descriptionArea:SetPosition( 15, 15 );
	self.descriptionArea:SetSize( 330, 170 );	
	self.descriptionArea:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	self.descriptionArea:SetForeColor(Turbine.UI.Color( 0, 0, 0));
	
	local scrollBar = Turbine.UI.Lotro.ScrollBar();
    scrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
    scrollBar:SetParent( background );
    scrollBar:SetPosition( self.descriptionArea:GetLeft()+self.descriptionArea:GetWidth(), self.descriptionArea:GetTop() );
    scrollBar:SetSize( 10, self.descriptionArea:GetHeight() );
	self.descriptionArea:SetVerticalScrollBar( scrollBar );	
end

function CampaignSelectorPane:CreateHeader()
	self.header = Turbine.UI.Control();
	self.header:SetParent( self );
	self.header:SetSize(378, 92);
	self.header:SetPosition((self:GetWidth()-378)/2, 2);	
	self.header:SetMouseVisible( false );
	self.header:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend )

	self.questName = Turbine.UI.Label();
	self.questName:SetParent( self );
	self.questName:SetSize(self:GetWidth()-30, 40);
	self.questName:SetPosition(15, 5);
	self.questName:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16);	
	self.questName:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.questName:SetMouseVisible( false );
	self.questName:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );

	self.authorLabel = Turbine.UI.Label();
	self.authorLabel:SetParent( self );
	self.authorLabel:SetSize(self:GetWidth(), 20);
	self.authorLabel:SetPosition(0, 95);
	self.authorLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	self.authorLabel:SetMouseVisible( false );

end


function CampaignSelectorPane:SetQuest(quest)
	self.quest = quest;
	if(quest == nil) then
		self:SetVisible(false);
		return;
	end
	self:SetVisible(true);
	
	local title="";
	if(quest.levelMin~=nil) then
		title = "["..quest.levelMin.."] ";
	end
	if(quest.title ~= nil) then
		title = title .. quest.title;
	else
		title = title .. "NO TITLE FOUND !";
	end
	self.questName:SetText( title );
	
	if(quest.header ~= nil) then
		self.header:SetBackground(quest.header);
	else
		self.header:SetBackground("WalmPlugins/YourOwnAdventures/pictures/campaigns/header_details2.tga");
	end
	
	local author = "";
	if(quest.author ~=nil) then
		author = _("BY").." "..quest.author.." ";
	end
	if(quest.version ~=nil) then
		author = author.." ".._("VERSION").." "..quest.version;
	end
	self.authorLabel:SetText(author);
	
	self.descriptionArea:SetText(quest.description);
	self.descriptionArea:SetSelectionStart(1);
	self.descriptionArea:InsertText("");
	self.descriptionArea:SetSelectionLength(1);
	
	self.restartButton:SetEnabled(quest.canRestart==true);
	
	self.restartButton.MouseClick = function(sender, args)	
		if(PROGRESS.ADVENTURES[self.quest.uniqueID]~=nil) then
			PROGRESS.ADVENTURES[self.quest.uniqueID] = nil;
		end
		self.launchButton:SetText(_("STARTQUESTBUTTON"));
		SaveProgress();
		campaignLoader:UpdateTreeViewStatus();
		if(self.quest.deleteItemsAtRestart ~= false) then
			inventoryEngine:RemoveItemsFromAdventure(self.quest.uniqueID);
		end
	end
	
	self.shareButton:SetEnabled(quest.shareURL~= nil);
	self.shareButton.MouseClick = function(sender, args)
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>".._("THEQUEST")..quest.title.._("DOWNLOADABLE").."<rgb=#FF0000>"..quest.shareURL.."</rgb>");
	end
	
	if(PROGRESS.ADVENTURES[quest.uniqueID]==nil or PROGRESS.ADVENTURES[quest.uniqueID].stepId=="START") then
		self.launchButton:SetText(_("STARTQUESTBUTTON"));
	elseif(PROGRESS.ADVENTURES[self.quest.uniqueID].stepId == "END") then
		self.launchButton:SetText(_("FINISHEDQUESTBUTTON"));
	else
		self.launchButton:SetText(_("CONTINUEQUESTBUTTON"));
	end
end
