
ContextualQuickSlots = class( Turbine.UI.Window );

function GetContextualQuickSlotsInstance()
	if(instanceContextualQuickSlots==nil) then
		instanceContextualQuickSlots = ContextualQuickSlots();
	end
	return instanceContextualQuickSlots;
end

function ContextualQuickSlots:Constructor()
	Turbine.UI.Window.Constructor( self );
	
	self:SetSize(100, 42);
	if(Preferences.quickslots_x==nil or Preferences.quickslots_y==nil) then
		Preferences.quickslots_x = Preferences.quicklaunch_x+50;
		Preferences.quickslots_y = Preferences.quicklaunch_y+130;
	end
	self:SetPosition(Preferences.quickslots_x, Preferences.quickslots_y);

	self.slots = {};
	self.slots2 = {};

	self:BuildBackground();

	self.MouseDown = function( sender, args )
		if(args.Button == Turbine.UI.MouseButton.Left and Preferences.canMoveQuickLauncher) then
			sender.dragStartX = args.X;
			sender.dragStartY = args.Y;
			sender.dragging = true;
			sender.dragged = false;
		end
	end
	self.MouseUp = function( sender, args )
		if (args.Button == Turbine.UI.MouseButton.Left) then			
			if (sender.dragging) then
				sender.dragging = false;
			end
			Preferences.quickslots_x, Preferences.quickslots_y = self:GetPosition();
			SavePreferences();
			if(Preferences.OnUpdate ~= nil) then
				Preferences.OnUpdate();
			end
		end
	end
	self.MouseMove = function(sender,args)
		if ( sender.dragging ) then
			local left, top = self:GetPosition();
			self:SetPosition( left + ( args.X - sender.dragStartX ), top + args.Y - sender.dragStartY );
			sender.dragged = true;
			if (self:GetLeft() > Turbine.UI.Display.GetWidth() - 35) then
				self:SetLeft(Turbine.UI.Display.GetWidth() - 35);				
			end
			if (self:GetLeft() < 0) then
				self:SetLeft(0);				
			end			
			if (self:GetTop() > Turbine.UI.Display.GetHeight() - 35) then
				self:SetTop(Turbine.UI.Display.GetHeight() - 35);				
			end
			if (self:GetTop() < 0) then
				self:SetTop(0);				
			end			
		end
	end
	
	-- Hide UI when F12 is pressed, code copied from Chiran sources
	local hideUI = false;
	local wasVisible;
	self:SetWantsKeyEvents(true);
	self.KeyDown = function(sender, args)
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			sender:SetVisible( false ) 
		elseif (args.Action == 268435635) then
			if not hideUI then
				hideUI = true;
				if self:IsVisible() then
					wasVisible = true;
					self:SetVisible(false);					
				else
					wasVisible = false;
				end

			else
				hideUI = false;
				if wasVisible then
					self:SetVisible(true);				
				end
			end
		end
	end
end

function ContextualQuickSlots:BuildBackground()
	self.leftBackground = Turbine.UI.Label();
	self.leftBackground:SetParent( self );
	self.leftBackground:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/quickslots/contextualquickslot_left.tga" );
	self.leftBackground:SetPosition( 0, 0 );
	self.leftBackground:SetSize(42, 42);
	self.leftBackground:SetMouseVisible( false );
	self.leftBackground:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	
	self.midBackground = Turbine.UI.Label();
	self.midBackground:SetParent( self );
	self.midBackground:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/quickslots/contextualquickslot_mid.tga" );
	self.midBackground:SetPosition( 42, 0 );
	self.midBackground:SetSize(self:GetWidth()-42*2, 42);
	self.midBackground:SetMouseVisible( false );
	self.midBackground:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );	
	
	self.rightBackground = Turbine.UI.Label();
	self.rightBackground:SetParent( self );
	self.rightBackground:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/quickslots/contextualquickslot_right.tga" );
	self.rightBackground:SetPosition( self:GetWidth()-42, 0 );
	self.rightBackground:SetSize(42, 42);
	self.rightBackground:SetMouseVisible( false );
	self.rightBackground:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );		
	
end

function ContextualQuickSlots:AddQuickSlot(picture, object, phrase, cdMaxTime, onClick)
	local nbcomp = table.getn(self.slots)+1;
	self:SetWidth((nbcomp+2)*32+4*(nbcomp-1));
	self.midBackground:SetSize(self:GetWidth()-42*2, 42);
	self.rightBackground:SetPosition( self:GetWidth()-42, 0 );	

	local qs = Turbine.UI.Lotro.Quickslot();
	qs:SetParent(self);
	qs:SetSize(32, 32);
	qs:SetPosition( 20+self.midBackground:GetWidth(), 5);
	qs:SetVisible(true);	
	qs:SetShortcut(Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Alias, FromUTF8(phrase)));
	qs:SetAllowDrop(false);
	qs:SetEnabled(false);
	qs.MouseUp = function (o, a)
		self:OnCooldown(qs);
		if(onClick~=nil ) then
			onClick(object);
		end
	end

	table.insert(self.slots, qs);

	if(type(cdMaxTime)=="number" and cdMaxTime>2) then
		qs.cdMaxTime = cdMaxTime;
	else
		qs.cdMaxTime = 2;
	end

	local qsLbl = Turbine.UI.Control();
	qsLbl:SetParent(self);
	qsLbl:SetSize(qs:GetSize());
	qsLbl:SetPosition(qs:GetPosition());
	qsLbl:SetMouseVisible(false);
	qsLbl:SetZOrder(qs:GetZOrder()+1);
	table.insert(self.slots2, qsLbl);

	local shield = Turbine.UI.Label();
	shield:SetParent( qsLbl );
	shield:SetSize(32, 32);
	shield:SetPosition( 0, 0);
	shield:SetMouseVisible(false);
	shield:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	shield:SetBackground(picture or "WalmPlugins/YourOwnAdventures/pictures/inv/default.jpg");
	qs.shield = shield;
	
	local cooldownLabel = Turbine.UI.Label();
	cooldownLabel:SetParent( shield );
	cooldownLabel:SetSize(32, 0);
	cooldownLabel:SetPosition( 0, 0);
	cooldownLabel:SetMouseVisible(false);
	cooldownLabel:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	cooldownLabel:SetBackground("WalmPlugins/YourOwnAdventures/pictures/inv/inv_filtered_slot.tga");
	qs.cooldown = cooldownLabel;
	
	if(object.name~=nil or object.description~=nil) then
		shield.tooltip = GenericTooltip();
		if(object.name~=nil) then
			shield.tooltip:AddPictureHeader(picture, object.name);
		end
		if(object.description~=nil) then 
			shield.tooltip:AddLayer(object.description, Turbine.UI.Color(197/255, 179/255, 88/255), nil, Turbine.UI.Lotro.Font.Verdana16);
		end	
		shield.tooltip:AddLayer(TRAD("Délai d'attente : ", "Wait time : ")..qs.cdMaxTime.."s", Turbine.UI.Color(0.5, 0.5, 0.5), nil, Turbine.UI.Lotro.Font.Verdana16);		
		qs.MouseEnter = function (a, e)
			local x = Turbine.UI.Display.GetMouseX()+20;
			local y = Turbine.UI.Display.GetMouseY()+40;
			if( x + shield.tooltip:GetWidth() > Turbine.UI.Display.GetWidth()) then x=Turbine.UI.Display.GetMouseX()-shield.tooltip:GetWidth()-10; end
			if( y + shield.tooltip:GetHeight() > Turbine.UI.Display.GetHeight()) then y=Turbine.UI.Display.GetMouseY()-shield.tooltip:GetHeight()-10; end
			shield.tooltip:SetPosition(x, y);
			shield.tooltip:SetVisible(true);
		end
		shield.MouseEnter = qs.MouseEnter;
		qs.MouseLeave = function (a, e)
			shield.tooltip:SetVisible(false);
		end
		shield.MouseLeave = qs.MouseLeave;
	end
	qs.timerRemain = self.timerRemain;
	qs.lastCooldown = self.timerRemain;
	if(qs.lastCooldown~=nil) then 
		qs.shield:SetMouseVisible(true);
	end		
end

function ContextualQuickSlots:AddPictureSlot(picture, object, cdMaxTime, onclickFct)

	local nbcomp = table.getn(self.slots)+1;
	self:SetWidth((nbcomp+2)*32+4*(nbcomp-1));
	self.midBackground:SetSize(self:GetWidth()-42*2, 42);
	self.rightBackground:SetPosition( self:GetWidth()-42, 0 );	

	local icone = Turbine.UI.Label();
	icone:SetParent( self );
	icone:SetBackground(picture );
	icone:SetSize(32, 32);
	icone:SetPosition( 20+self.midBackground:GetWidth(), 5 );
	icone:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	icone.MouseClick = function(sender, args)
		if(icone.cdUp==true) then return; end
		self:OnCooldown(icone);
		onclickFct(object);
	end
	
	if(type(cdMaxTime)=="number" and cdMaxTime>2) then
		icone.cdMaxTime = cdMaxTime;
	else
		icone.cdMaxTime = 2;
	end	
	
	local cooldownLabel = Turbine.UI.Label();
	cooldownLabel:SetParent( icone );
	cooldownLabel:SetSize(32, 0);
	cooldownLabel:SetPosition( 0, 0);
	cooldownLabel:SetMouseVisible(false);
	cooldownLabel:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	cooldownLabel:SetBackground("WalmPlugins/YourOwnAdventures/pictures/inv/inv_filtered_slot.tga");
	icone.cooldown = cooldownLabel;	
	if(object.name~=nil or object.description~=nil) then
		icone.tooltip = GenericTooltip();
		if(object.name~=nil) then
			icone.tooltip:AddPictureHeader(picture, object.name);
		end
		if(object.description~=nil) then 
			icone.tooltip:AddLayer(object.description, Turbine.UI.Color(197/255, 179/255, 88/255), nil, Turbine.UI.Lotro.Font.Verdana16);
		end	
		icone.tooltip:AddLayer(TRAD("Délai d'attente : ", "Wait time : ")..icone.cdMaxTime.."s", Turbine.UI.Color(0.5, 0.5, 0.5), nil, Turbine.UI.Lotro.Font.Verdana16);
		icone.MouseEnter = function (a, e)
			local x = Turbine.UI.Display.GetMouseX()+20;
			local y = Turbine.UI.Display.GetMouseY();
			if( x + icone.tooltip:GetWidth() > Turbine.UI.Display.GetWidth()) then x=Turbine.UI.Display.GetMouseX()-icone.tooltip:GetWidth()-10; end
			if( y + icone.tooltip:GetHeight() > Turbine.UI.Display.GetHeight()) then y=Turbine.UI.Display.GetMouseY()-icone.tooltip:GetHeight()-10; end
			icone.tooltip:SetPosition(x, y);
			icone.tooltip:SetVisible(true);
		end
		icone.MouseLeave = function (a, e)
			icone.tooltip:SetVisible(false);
		end
	end
	icone.timerRemain = self.timerRemain;
	icone.lastCooldown = self.timerRemain;
	if(icone.lastCooldown~=nil) then 
		icone.cdUp = true;
	end
	table.insert(self.slots, icone);
end

function ContextualQuickSlots:OnCooldown(parent)
	for i=1, table.getn(self.slots), 1 do
		local cpt = self.slots[i];
		if(cpt==parent)then
			cpt.timerRemain = cpt.cdMaxTime;
			cpt.lastCooldown = cpt.cdMaxTime;
		elseif(cpt.timerRemain==nil or cpt.timerRemain<2) then
			cpt.timerRemain = 2;
			cpt.lastCooldown = 2;
		end
		if(cpt.shield~=nil) then 
			cpt.shield:SetMouseVisible(true);
		else
			cpt.cdUp = true;
		end
	end		
	self.timerRemain = 2;
	self.lastUpdate = 0;
	self:SetWantsUpdates(true);
end

function ContextualQuickSlots:Update( args )
	local delta = 0;
	local currentTime = Turbine.Engine.GetGameTime();

	if ( self.lastUpdate > 0 ) then
		delta = currentTime - self.lastUpdate;
	else	
		self.timerRemain = 2;
	end

	self.lastUpdate = currentTime;

	self.timerRemain = self.timerRemain - delta;
	local count = 0;
	for i=1, table.getn(self.slots), 1 do
		local cpt = self.slots[i];
		if(cpt.timerRemain~=nil and cpt.timerRemain>0) then
			cpt.timerRemain = cpt.timerRemain - delta;
			if(cpt.timerRemain<=0) then
				cpt.timerRemain = 0;
				if(cpt.shield~=nil) then 
					cpt.shield:SetMouseVisible(false);
				else
					cpt.cdUp = nil;
				end	
			else
				count=count+1;
			end
			local h = 32.0-32.0*(cpt.lastCooldown-cpt.timerRemain)/cpt.lastCooldown;
			--Turbine.Shell.WriteLine("<rgb=#FF000>DELTA = "..delta.." REMAINS "..cpt.timerRemain.." h="..h.." current="..currentTime);	
			cpt.cooldown:SetHeight(h);
		end
	end
	if(count==0 and self.timerRemain<=0) then
		self:SetWantsUpdates(false);
		self.timerRemain = nil;
	end
end

function ContextualQuickSlots:Reset()
	for i=1, table.getn(self.slots), 1 do
		local cpt = table.remove(self.slots, 1);
		cpt:SetVisible(false);
		cpt:SetParent(nil);
		cpt = nil;
	end	
	for i=1, table.getn(self.slots2), 1 do
		local cpt = table.remove(self.slots2, 1);
		cpt:SetVisible(false);
		cpt:SetParent(nil);
		cpt = nil;
	end	
end

function ContextualQuickSlots:GetName()
	return "ContextualQuickSlots";
end

function ContextualQuickSlots:IsRunning()
	return ( self ~= nil );
end

function ContextualQuickSlots:Start()
	if ( self ~= nil ) then
		self:SetVisible( true );
		return;
	end	
end

function ContextualQuickSlots:Stop()
	self:SetVisible(false);
	self = nil;
end