DiaryFamilyNode = class( Turbine.UI.TreeNode );

function DiaryFamilyNode:Constructor(width, familyName)
	self.familyName = familyName;
	Turbine.UI.TreeNode.Constructor( self );    	
	self:SetSize( width, 42 );
	self:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	
	self.labelValue = Turbine.UI.Label();
	self.labelValue:SetParent( self );
	self.labelValue:SetSize( width-35, 22 );
	self.labelValue:SetPosition( 32, 0 );
	self.labelValue:SetFont( Turbine.UI.Lotro.Font.BookAntiquaBold22 );	
	self.labelValue:SetText(FormatAdventureString(familyName));
	self.labelValue:SetForeColor( Turbine.UI.Color(0,0,0) );
	self.labelValue:SetMouseVisible( false );

	self.checkBox = Turbine.UI.Control();
	self.checkBox:SetParent( self );
	self.checkBox:SetSize( 25, 25 );
	self.checkBox:SetPosition( 0, 6 );
	self.checkBox:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	self.checkBox:SetMouseVisible( false );
	self.checkBox:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/diary/checkbox"..math.random(1, 3)..".tga" );
	
	
	self.MouseEnter = function (sender, args)
		self.labelValue:SetForeColor( Turbine.UI.Color(0.12,0,0.83) );
	end
	self.MouseLeave = function (sender, args)
		self.labelValue:SetForeColor( Turbine.UI.Color(0,0,0) );
	end	
	
	self.dashList = {};
	self:UpdateData();
end

function DiaryFamilyNode:UpdateData()
	local adv = 0;
	local finished = 0;
	for i=1, table.getn(ADVENTURES), 1 do
		local family = ADVENTURES[i].family or _("ORPHANS");
		if(family == self.familyName) then
			adv = adv + 1;
			if(PROGRESS.ACHIEVEMENTS[ADVENTURES[i].uniqueID] ~= nil) then
				if(PROGRESS.ACHIEVEMENTS[ADVENTURES[i].uniqueID].count>0) then
					finished = finished + 1;
				end
			end
		end
	end
	
	if(adv==finished) then
		self.checkBox:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/diary/checkbox"..math.random(1, 3).."_checked.tga" );
	else
		self.checkBox:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/diary/checkbox"..math.random(1, 3)..".tga" );
	end
	
	for i, label in pairs(self.dashList) do
		label:SetParent(nil);
		label = nil;
	end
	local left = 32;
	local sectSep = 10;
	local fin = finished;
	if (finished>20) then fin = 20; end
	for i=1, fin, 1 do
		local id = math.fmod(i-1, 5);
		local section = math.floor((i-1)/5);
		local label = Turbine.UI.Label();
		label:SetParent( self );
		label:SetMouseVisible( false );
		label:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
		if(id<4) then
			label:SetSize( 8, 18 );
			label:SetPosition( left+id*8+section*48, 19 );
			label:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/diary/dash"..math.random(1, 5)..".tga" );
		else
			label:SetSize( 40, 8 );
			label:SetPosition( left+section*48, 24 );	
			label:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/diary/dash_h1.tga" );			
		end
		table.insert(self.dashList, label);
	end
	if (finished>20) then
		plusLabel = Turbine.UI.Label();
		plusLabel:SetParent( self );
		plusLabel:SetSize( 40, 22 );
		plusLabel:SetPosition( self:GetWidth()-40, 18 );
		plusLabel:SetFont( Turbine.UI.Lotro.Font.BookAntiqua24 );	
		plusLabel:SetText("+"..(finished-20));
		plusLabel:SetForeColor( Turbine.UI.Color(0,0,0) );
		plusLabel:SetMouseVisible( false );
		table.insert(self.dashList, label);
	end
	
	local updated = not (self.advCount==adv and self.advFinished==finished);
	self.advCount = adv;
	self.advFinished = finished;
	return updated;
end

function DiaryFamilyNode:ExpandAll()
	self:SetExpanded(true);
	for i=1, self:GetChildNodes():GetCount(), 1 do
		self:GetChildNodes():Get(i):ExpandAll();
	end
end

function DiaryFamilyNode:CollapseAll()
	self:SetExpanded(false);
	for i=1, self:GetChildNodes():GetCount(), 1 do
		self:GetChildNodes():Get(i):CollapseAll();
	end	
end