

ObjectivesLabel = class( Turbine.UI.TreeView );

function ObjectivesLabel:Constructor(width)
	Turbine.UI.TreeView.Constructor( self );    	
	self:SetSize(width, 0);
	self:SetMouseVisible( false );	
	self:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/launcher/bandeau.jpg" );	
	self:SetVisible(false);
end

function ObjectivesLabel:AutoResize()
	local rootNodes = self:GetNodes();
	local height = 0;
	for i=1, rootNodes:GetCount(), 1 do
		node = rootNodes:Get(i);
		height = height + node:GetHeight();
		for j=1, node:GetChildNodes():GetCount(), 1 do
			node2 = node:GetChildNodes():Get(j);
			height = height + node2:GetHeight();
		end
		node:ExpandAll();
	end	
	self:SetHeight(height);
	return height;
end

function ObjectivesLabel:AddObjective(objectives)
	if(objectives == nil) then return nil; end
	
	local objectiveNode = DiaryEntryNode(self:GetWidth(), 5, 5);
	objectiveNode:SetFore(Turbine.UI.Color(1, 1, 1), Turbine.UI.Lotro.Font.Verdana12, Turbine.UI.ContentAlignment.TopCenter);
	objectiveNode.objectives = objectives;
	local upperSep = Turbine.UI.Control();
	upperSep:SetParent( objectiveNode );
	upperSep:SetSize( self:GetWidth()-20 , 2 );
	upperSep:SetPosition( 10, 0 );	
	upperSep:SetBackColor( Turbine.UI.Color(0.5, 0.5, 0.5) );
	
	local objectiveLabel = self:GetObjectivesName(objectives);
	objectiveNode:UpdateText(objectiveLabel);
	self:GetNodes():Add(objectiveNode);
	return objectiveLabel;
end

function ObjectivesLabel:GetObjectivesName(objectives, objective, progress, goal)
	if(objectives==ni) then return ""; end
	local objectiveLabel = "";
	if(objectives.step~=nil) then
		local add="";
		if(objectives.canUpdateProgress==true) then
			add=" "..objectives:GetProgress();
		end
		if(objectives.step.objectif~=nil) then
			objectiveLabel =  objectives.step.objectif..add;
		end
	else
		local passed = false;
		for i, step in pairs(objectives) do
			if(step.step~=nil) then
				if(step.step.objectif~=nil) then
					local add="";
					if(step.step.objectif.canUpdateProgress==true) then
						add=objective:GetProgress();
					end					
					if(passed==true) then
						objectiveLabel = objectiveLabel..TRAD("\nou\n", "\nor\n");
					end
					objectiveLabel =  objectiveLabel..step.step.objectif..add;
					passed = true;
				end
			end
		end
	end
	return objectiveLabel;
end 

function ObjectivesLabel:RemoveObjectives()
	self:GetNodes():Clear();
	self:SetVisible(false);
end

function ObjectivesLabel:RemoveObjective(objective)
	local rootNodes = self:GetNodes();
	local nodeToRemove = nil;
	for i=1, rootNodes:GetCount(), 1 do
		local node = rootNodes:Get(i);
		if(node~=nil and (node.objectives==objective or node.objectives=={objective})) then
			nodeToRemove = node;
			break;
		elseif(node~=nil and type(node.objectives)=="table") then
			for i, obj in pairs(node.objectives) do
				if(obj == objective) then
					nodeToRemove = node;
					break;
				end
			end
		end
	end
	if(nodeToRemove~=nil) then
		rootNodes:Remove(nodeToRemove);
	end
end


function ObjectivesLabel:SetObjectiveProgress(objective, progress, goal)	
	local rootNodes = self:GetNodes();
	local nodeToEdit = nil;
	for i=1, rootNodes:GetCount(), 1 do
		local node = rootNodes:Get(i);
		if(node~=nil and (node.objectives==objective or node.objectives=={objective})) then
			nodeToEdit = node;
			break;
		elseif(node~=nil and type(node.objectives)=="table") then
			for i, obj in pairs(node.objectives) do
				if(obj == objective) then
					nodeToEdit = node;
					break;
				end
			end
		end
	end
	if(nodeToEdit~=nil) then
		local objName = self:GetObjectivesName(nodeToEdit.objectives, objective, progress, goal);
		nodeToEdit:UpdateText(objName);
	end
end

function ObjectivesLabel:ShowSuggestedAdventures(lastAdventure)
	
	if(AdventureEngine.quest~=nil or lastAdventure==nil) then return; end
	local unlockedNode = DiaryEntryNode(self:GetWidth(), 5, 5);
	local suggestedNode = DiaryEntryNode(self:GetWidth(), 5, 5);
	for i, adv in pairs(ADVENTURES) do
		if(adv.requireCompletion~=nil) then
			if(adv.requireCompletion==lastAdventure.uniqueID) then
				if(PROGRESS.ACHIEVEMENTS[adv.uniqueID]==nil or PROGRESS.ACHIEVEMENTS[adv.uniqueID].firstCompletionDate==nil) then
					unlockedNode:GetChildNodes():Add(self:BuildSuggestionNode(adv, unlockedNode:GetChildNodes():GetCount()>0));
				end
			elseif (type(adv.requireCompletion)=="table" and ListContainsString(adv.requireCompletion, lastAdventure.uniqueID)) then
				local islocked = false;
				for j, id in pairs(adv.requireCompletion) do
					if(PROGRESS.ACHIEVEMENTS[id]==nil or PROGRESS.ACHIEVEMENTS[id].firstCompletionDate==nil) then
						islocked = true;
						suggestedNode:GetChildNodes():Add(self:BuildSuggestionNode(AdventureEngine:GetAdventureById(id), suggestedNode:GetChildNodes():GetCount()>0));
					end
				end
				if(islocked==false) then
					unlockedNode:GetChildNodes():Add(self:BuildSuggestionNode(adv, unlockedNode:GetChildNodes():GetCount()>0));
				end
			end
		end
	end
	if(unlockedNode:GetChildNodes():GetCount()>0) then
		unlockedNode:SetFore(Turbine.UI.Color(1, 1, 1), Turbine.UI.Lotro.Font.Verdana12, Turbine.UI.ContentAlignment.TopCenter);
		if(unlockedNode:GetChildNodes():GetCount()==1) then
			unlockedNode:UpdateText(TRAD("Nouvelle aventure débloquée :", "New adventure unlocked :"));
		else
			unlockedNode:UpdateText(TRAD("Nouvelles aventures débloquées :", "New adventures unlocked :"));
		end
		self:BuildHeaderNode(unlockedNode);
		self:GetNodes():Add(unlockedNode);	
	end
	if(suggestedNode:GetChildNodes():GetCount()>0) then
		suggestedNode:SetFore(Turbine.UI.Color(1, 1, 1), Turbine.UI.Lotro.Font.Verdana12, Turbine.UI.ContentAlignment.TopCenter);
		if(suggestedNode:GetChildNodes():GetCount()==1) then
			suggestedNode:UpdateText(TRAD("Aventure suggérée :", "Suggested adventure :"));
		else
			suggestedNode:UpdateText(TRAD("Aventures suggérées :", "Suggested adventures :"));
		end
		self:BuildHeaderNode(suggestedNode);
		self:GetNodes():Add(suggestedNode);	
	end
	-- if(self:GetNodes():GetCount()==0) then
		-- local headerNode = DiaryEntryNode(self:GetWidth(), 5, 5);
		-- headerNode:SetFore(Turbine.UI.Color(1, 1, 1), Turbine.UI.Lotro.Font.Verdana12, Turbine.UI.ContentAlignment.TopCenter);
		-- headerNode:UpdateText(TRAD("Nouvelle aventure(s) débloquée(s) :", "New adventure(s) unlocked :"));
		-- self:GetNodes():Add(headerNode);
	-- end
	--TODO create child
end

function ObjectivesLabel:BuildHeaderNode(node)
	node:SetHeight(24);
	local leftIcone = Turbine.UI.Control();
	leftIcone:SetParent( node );
	leftIcone:SetSize( 16, 24 );
	leftIcone:SetPosition(0, 0);
	leftIcone:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	leftIcone:SetBackground("WalmPlugins/YourOwnAdventures/pictures/launcher/suggest/HeaderNode_Left.tga");	
	local rightIcone = Turbine.UI.Control();
	rightIcone:SetParent( node );
	rightIcone:SetSize( 16, 24 );
	rightIcone:SetPosition(node:GetWidth()-16, 0);
	rightIcone:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	rightIcone:SetBackground("WalmPlugins/YourOwnAdventures/pictures/launcher/suggest/HeaderNode_Right.tga");	
	local centerIcone = Turbine.UI.Control();
	centerIcone:SetParent( node );
	centerIcone:SetSize( node:GetWidth()-16-16, 24 );
	centerIcone:SetPosition(16, 0);
	centerIcone:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	centerIcone:SetBackground("WalmPlugins/YourOwnAdventures/pictures/launcher/suggest/HeaderNode_Mid.tga");
	node.labelValue:SetZOrder(centerIcone:GetZOrder()+1);
end

function ObjectivesLabel:BuildSuggestionNode(adventure, showSep)
	local newNode = DiaryEntryNode(self:GetWidth(), 5, 5, 30, 5);
	newNode:SetFore(Turbine.UI.Color(1, 1, 1), Turbine.UI.Lotro.Font.Verdana12, Turbine.UI.ContentAlignment.TopCenter);
	newNode:UpdateText(adventure.title, true);
	newNode:SetMouseVisible(true);
	newNode.MouseClick = function (a, o)
		campaignLoader:SelectAdventure(adventure);
	end	
	local linkIcone = Turbine.UI.Control();
	linkIcone:SetParent( newNode );
	linkIcone:SetSize( 16, 16 );
	linkIcone:SetPosition( newNode:GetWidth()-24, (newNode:GetHeight()-16)/2 );
	linkIcone:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	linkIcone:SetBackground("WalmPlugins/YourOwnAdventures/pictures/launcher/suggest/next_arrow.tga");
	linkIcone.MouseClick = function (a, o)
		campaignLoader:SelectAdventure(adventure);
	end
	if(showSep==true) then
		local upperSep = Turbine.UI.Control();
		upperSep:SetParent( newNode );
		upperSep:SetSize( self:GetWidth()-20 , 2 );
		upperSep:SetPosition( 10, 0 );	
		upperSep:SetBackColor( Turbine.UI.Color(0.5, 0.5, 0.5) );
	end
	return newNode;
end