
import "WalmPlugins.YourOwnAdventures.CampaignSelector";
import "WalmPlugins.YourOwnAdventures.AdventuresDiary";
import "WalmPlugins.YourOwnAdventures.PlayerSheetWindow";
import "WalmPlugins.YourOwnAdventures.HelpWindow";
import "WalmPlugins.YourOwnAdventures.MenuButton";
import "WalmPlugins.YourOwnAdventures.LerpValue";
import "WalmPlugins.YourOwnAdventures.TimerLabel";
import "WalmPlugins.YourOwnAdventures.ObjectivesLabel";

QuickLauncher = class( Turbine.UI.Window );

function QuickLauncher:Constructor(dialog)
	Turbine.UI.Window.Constructor( self );	
	--self.defaultOpacity = 0.6;
	self:SetSize(275, 83);
	if(Preferences.quicklaunch_x==nil or Preferences.quicklaunch_y==nil) then
		local width, height = Turbine.UI.Display.GetSize();		
		Preferences.quicklaunch_x = (width-self:GetWidth())/2;
		Preferences.quicklaunch_y = 100;
		
	end
	self:SetPosition(Preferences.quicklaunch_x, Preferences.quicklaunch_y);
	self:SetOpacity(Preferences.defaultOpacity / 100);
	
	self:SetMouseVisible( true );
	
	self.background = Turbine.UI.Label();
	self.background:SetParent( self );
	self.background:SetBackground( "WalmPlugins/YourOwnAdventures/pictures/launcher/quicklauncher2.tga" );
	self.background:SetPosition( 0, 18 );
	self.background:SetSize(275, 65);
	self.background:SetMouseVisible( false );
	self.background:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );	
	
	self.questName = Turbine.UI.Label();
	self.questName:SetParent( self );
	self.questName:SetText( "" );
	self.questName:SetPosition(55, 0);
	self.questName:SetSize(215, 32);
	self.questName:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16); 
	self.questName:SetFontStyle(Turbine.UI.FontStyle.Outline); 
	self.questName:SetTextAlignment( Turbine.UI.ContentAlignment.BottomLeft ); 
	self.questName:SetMouseVisible( false );	
	self.questName:SetForeColor( Turbine.UI.Color(207/255,181/255,59/255) );
	
	self.objectivesLabel = ObjectivesLabel(199);
	self.objectivesLabel:SetParent( self );
	self.objectivesLabel:SetPosition(30, 65);
	self.objectivesLabel:SetZOrder(self.background:GetZOrder()-1);
	
	self.timerComponent = TimerLabel();
	self.timerComponent:SetParent( self );
	
	
	self:BuildButtons(dialog);
	
	self.MouseEnter = function(sender,args)
		self:SetOpacity(1.0);
	end
	self.MouseLeave = function(sender,args)
		self:SetOpacity((Preferences.defaultOpacity or 60) / 100);
	end		
	self.MouseDown = function( sender, args )
		if(args.Button == Turbine.UI.MouseButton.Left and Preferences.canMoveQuickLauncher) then
			sender.dragStartX = args.X;
			sender.dragStartY = args.Y;
			sender.dragging = true;
			sender.dragged = false;
		end
	end
	self.MouseUp = function( sender, args )
		if (args.Button == Turbine.UI.MouseButton.Left) then			
			if (sender.dragging) then
				sender.dragging = false;
			end
			Preferences.quicklaunch_x, Preferences.quicklaunch_y = self:GetPosition();
			SavePreferences();
			if(Preferences.OnUpdate ~= nil) then
				Preferences.OnUpdate();
			end
		end
	end
	self.MouseMove = function(sender,args)
		if ( sender.dragging ) then
			local left, top = self:GetPosition();
			self:SetPosition( left + ( args.X - sender.dragStartX ), top + args.Y - sender.dragStartY );
			sender.dragged = true;
			if (self:GetLeft() > Turbine.UI.Display.GetWidth() - 35) then
				self:SetLeft(Turbine.UI.Display.GetWidth() - 35);				
			end
			if (self:GetLeft() < 0) then
				self:SetLeft(0);				
			end			
			if (self:GetTop() > Turbine.UI.Display.GetHeight() - 35) then
				self:SetTop(Turbine.UI.Display.GetHeight() - 35);				
			end
			if (self:GetTop() < 0) then
				self:SetTop(0);				
			end			
		end
	end
	
	self:ApplyPreferences();
	
	-- Hide UI when F12 is pressed, code copied from Chiran sources
	local hideUI = false;
	local wasVisible;
	self:SetWantsKeyEvents(true);
	self.KeyDown = function(sender, args)
		if (args.Action == 268435635) then
			if not hideUI then
				hideUI = true;
				if self:IsVisible() then
					wasVisible = true;
					self:SetVisible(false);					
				else
					wasVisible = false;
				end

			else
				hideUI = false;
				if wasVisible then
					self:SetVisible(true);				
				end
			end
		end
	end
end

function QuickLauncher:AddObjective(objectives)
	if(objectives == nil) then return; end
	
	local objectiveStr = self.objectivesLabel:AddObjective(objectives);
	
	self:ReplaceComponents();
	if(objectiveStr~=nil) then
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>"..TRAD("Nouvel objectif : ", "New objective : ").."<rgb=#FFFFFF>"..string.gsub(objectiveStr, "\n", " "));
	end
end

function QuickLauncher:RemoveObjectives()
	self.objectivesLabel:RemoveObjectives();
	self:ReplaceComponents();
end

function QuickLauncher:RemoveObjective(objective)
	self.objectivesLabel:RemoveObjective(objective);
	self:ReplaceComponents();
end

function QuickLauncher:SetObjectiveProgress(objective, progress, goal)	
	self.objectivesLabel:SetObjectiveProgress(objective, progress, goal);
	self:ReplaceComponents();
end

function QuickLauncher:ShowSuggestedAdventures(lastAdventure)
	self.objectivesLabel:ShowSuggestedAdventures(lastAdventure);
	self:ReplaceComponents();
end

function QuickLauncher:ReplaceComponents()
	local height = self.objectivesLabel:AutoResize();
	self.objectivesLabel:SetVisible(self.objectivesLabel:GetNodes():GetCount()>0);
	if(self.timerComponent.timer==nil) then 
		self:SetHeight(height+self.objectivesLabel:GetTop()); 
		self.timerComponent:SetPosition(self.objectivesLabel:GetLeft()+self.objectivesLabel:GetWidth()/2-50, self:GetHeight()-20);
	else
		self:SetHeight(height+self.objectivesLabel:GetTop()+17); 
		self.timerComponent:SetPosition(self.objectivesLabel:GetLeft()+self.objectivesLabel:GetWidth()/2-50, self:GetHeight()-20);
	end
	if(self:GetHeight()<83) then self:SetHeight(83); end
end

function QuickLauncher:BuildButtons(dialog)
	local top = 18;
	local left = 60;
	local dec = 35;

	self.questLauncher = MenuButton("WalmPlugins/YourOwnAdventures/pictures/launcher/q_quest.tga");
	self.questLauncher:SetParent( self.background );
	self.questLauncher:SetPosition( left, top );
	self.questLauncher.tooltip = GenericTooltip(self.questLauncher, _("TOOLTIP_QUEST1"), _("TOOLTIP_QUEST2"));
	self.questLauncher:AddCounter(10);
	self.questLauncher.MouseClick = function(sender, args)
		self.questLauncher:RemoveAlert(false);
		dialog:UpdateSelectedNode();
		dialog:SetVisible( not dialog:IsVisible() );
	end
	
	self.questDialog = MenuButton("WalmPlugins/YourOwnAdventures/pictures/launcher/q_journal.tga");
	self.questDialog:SetParent( self.background );
	self.questDialog:SetPosition( left+dec, top );
	AdvDiary:SetMenuButton(self.questDialog);
	self.questDialog.tooltip = GenericTooltip(self.questDialog, _("TOOLTIP_QUESTDIAL1"), _("TOOLTIP_QUESTDIAL2"));
	self.questDialog.MouseClick = function(sender, args)
		self.questDialog:RemoveAlert(false);
		AdvDiary:ShowDialog( not AdvDiary:IsVisible() , nil);
	end	
	
	self.invDialog = MenuButton("WalmPlugins/YourOwnAdventures/pictures/launcher/q_inventory.tga");
	self.invDialog:SetParent( self.background );
	self.invDialog:SetPosition( left+dec*2, top );
	inventory:SetMenuButton(self.invDialog);
	self.invDialog.tooltip = GenericTooltip(self.invDialog, _("TOOLTIP_INV1"), _("TOOLTIP_INV2"));
	self.invDialog.MouseClick = function(sender, args)
		self.invDialog:RemoveAlert(false);
		inventory:SetVisible( not inventory:IsVisible() );
	end	

	self.succesDialog = MenuButton("WalmPlugins/YourOwnAdventures/pictures/launcher/q_title.tga");
	self.succesDialog:SetParent( self.background );
	self.succesDialog:SetPosition( left+dec*3, top );
	playersheet:SetMenuButton(self.succesDialog);
	self.succesDialog.tooltip = GenericTooltip(self.succesDialog, _("TOOLTIP_EXPLOIT1"), _("TOOLTIP_EXPLOIT2"));
	self.succesDialog.MouseClick = function(sender, args)
		self.succesDialog:RemoveAlert(false);
		playersheet:SetVisible( not playersheet:IsVisible() );
	end	
	
	
	self.optionsDialog = MenuButton("WalmPlugins/YourOwnAdventures/pictures/launcher/q_help.tga");
	self.optionsDialog:SetParent( self.background );
	self.optionsDialog:SetPosition( left+dec*4, top );
	self.optionsDialog.tooltip = GenericTooltip(self.optionsDialog, _("TOOLTIP_OPTIONS1"), _("TOOLTIP_OPTIONS2"));

	local helpWindow=nil;
	self.optionsDialog.MouseClick = function(sender, args)
		if(helpWindow==nil) then
			helpWindow = HelpWindow();
		end
		helpWindow:SetVisible( not helpWindow:IsVisible());
	end		

end

function QuickLauncher:InitAdventureTimer(timer, step)
	self:SetHeight(self.objectivesLabel:GetTop()+self.objectivesLabel:GetHeight()+17);
	self.timerComponent:SetPosition(self.objectivesLabel:GetLeft()+self.objectivesLabel:GetWidth()/2-50, self:GetHeight()-20);
	self.timerComponent:InitAdventureTimer(timer, step);
end

function QuickLauncher:GetAdventureTimer()
	return self.timerComponent:GetAdventureTimer();
end

function QuickLauncher:StopAdventureTimer(timer)
	self.timerComponent:StopAdventureTimer(timer);
	self:ReplaceComponents()
end

function QuickLauncher:ApplyPreferences()
	self:SetPosition(Preferences.quicklaunch_x, Preferences.quicklaunch_y);
	self:SetOpacity(Preferences.defaultOpacity / 100);
end

function QuickLauncher:GetName()
	return "QuickLauncher";
end

function QuickLauncher:IsRunning()
	return ( self ~= nil );
end

function QuickLauncher:Start()
	if ( self ~= nil ) then
		self:SetVisible( true );
		return;
	end	
end

function QuickLauncher:Stop()
	self:SetVisible(false);
	self = nil;
end