StepReactorKILL = class();

function StepReactorKILL:Constructor(adventure, stepId)
	self.canUpdateProgress = true;
	self.step = adventure.steps[stepId];
	self.stepId = stepId;
	if(self.step==nil) then
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>YourOwnAdventures: ".._("STEP_ERR_UNK").."</rgb>");
		return false;
	end
	self.adventure = adventure;
	
	local loadingCounter = 0;
	if(PROGRESS.ADVENTURES[adventure.uniqueID]~=nil and PROGRESS.ADVENTURES[adventure.uniqueID][stepId]~=nil) then
		if(PROGRESS.ADVENTURES[adventure.uniqueID][stepId].kill~=nil) then
			loadingCounter = PROGRESS.ADVENTURES[adventure.uniqueID][stepId].kill or 0;
			quickLauncher:SetObjectiveProgress(self, loadingCounter, self.step.count or 1);
		end
	end	
	
	self:InitChatLogger(loadingCounter);
	
	if(self.step.count == nil) then
		self.step.count = 1;
	end
end

function StepReactorKILL:GetProgress()
	local loadingCounter = 0;
	if(PROGRESS.ADVENTURES[self.adventure.uniqueID]~=nil and PROGRESS.ADVENTURES[self.adventure.uniqueID][self.stepId]~=nil) then
		if(PROGRESS.ADVENTURES[self.adventure.uniqueID][self.stepId].kill~=nil) then
			loadingCounter = PROGRESS.ADVENTURES[self.adventure.uniqueID][self.stepId].kill or 0;
		end
	end	
	return loadingCounter.."/"..(self.step.count or 1);
end

function StepReactorKILL:InitChatLogger(loadingCounter)
	self.killCount = loadingCounter;
	function StepReactorKILLtriggerCompletion(trigger, message)  
		if(trigger.parent~=nil and trigger.parent.OnTrigger~=nil) then
			trigger.parent:OnTrigger(trigger, message);
		end
	end
	if(type(self.step.tracker)=="string" ) then
		chatlogger:AddTrigger(Turbine.ChatType.Death, self.step.tracker, StepReactorKILLtriggerCompletion, nil, self);
	elseif(type(self.step.tracker)=="table") then
		self.trigger = {};
		for i, j in pairs(self.step.tracker) do
			chatlogger:AddTrigger(Turbine.ChatType.Death, j, StepReactorKILLtriggerCompletion, nil, self);
		end		
	end
end

function StepReactorKILL:OnTrigger(trigger, message)
	if(self.killCount>=self.step.count) then
		self:Completed();
		return;
	end
	
	if(self.step.doNotCountKills~=true) then
		self.killCount = self.killCount + 1;
	end
	
	if(self.step.doOnEachKill~=nil) then
		for i, doIt in pairs(self.step.doOnEachKill) do 
			local exectute=true;
			if (doIt.probability~=nil) then
				exectute = exectute and (math.random(100)<doIt.probability);
			end
			if (doIt.requireItem~=nil) then
				exectute = exectute and inventoryEngine:HasItem(doIt.requireItem);
			end
			if(exectute==true) then
				if(type(doIt.completionIncCounter)=="number") then
					self.killCount = self.killCount + doIt.completionIncCounter;
				end					
				if(doIt.completionIncMessage~=nil) then
					doIt.pushSystemChat = doIt.completionIncMessage.." "..self.killCount.."/"..self.step.count;
				end
				AdventureEngine:ApplyStepGenericActions(doIt);
			end
			if(PROGRESS.ADVENTURES[self.adventure.uniqueID] == nil) then
				PROGRESS.ADVENTURES[self.adventure.uniqueID] = {};
			end
			if(PROGRESS.ADVENTURES[self.adventure.uniqueID][self.stepId]==nil) then
				PROGRESS.ADVENTURES[self.adventure.uniqueID][self.stepId] = {};
			end
			PROGRESS.ADVENTURES[self.adventure.uniqueID][self.stepId].kill=self.killCount; 
			SaveProgress();
		end
	end

	quickLauncher:SetObjectiveProgress(self, self.killCount, self.step.count);
	if(self.killCount>=self.step.count) then
		self:Completed();
	end
end

function StepReactorKILL:Completed()
	if(AdventureEngine:TestSuccessCondition(self.step.successCondition)==false) then
		return;
	end
	if(PROGRESS.ADVENTURES[self.adventure.uniqueID]~=nil and PROGRESS.ADVENTURES[self.adventure.uniqueID][self.stepId]~=nil) then
		PROGRESS.ADVENTURES[self.adventure.uniqueID][self.stepId].kill = 0;
	end
	SaveProgress();
	AdventureEngine:ApplyStepGenericActions(self.step.onCompletion);
	AdventureEngine:ClearSynchronousReactors(self);
end

function StepReactorKILL:Unload()
	
	chatlogger:RemoveTrigger(Turbine.ChatType.Death, self.step.tracker);
	--chatlogger:ClearTriggers();
	--Turbine.Shell.WriteLine("<rgb=#FF5CCD>UNLOAD ");
end