import "WalmPlugins.YourOwnAdventures.TalkTree";

StepReactorTALKTREE = class();

function StepReactorTALKTREE:Constructor(adventure, stepId)
	self.step = adventure.steps[stepId];
	self.stepId = stepId;
	if(self.step==nil) then
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>YourOwnAdventures: ".._("STEP_ERR_UNK").."</rgb>");
		return false;
	end
	
	self:InitTargetLogger();
	self.talkTree = TalkTree();

	if(self.step.autostart==true) then
		self:TargetChanged();
	end
end

function StepReactorTALKTREE:TargetChanged(sender, args)
	if target~=nil then
		local targetname = target:GetName();
		
		if(self.step.requireExactName==true and type(self.step.tracker)=="string" and targetname==self.step.tracker) then
			self:Completed();
		elseif((self.step.requireExactName==nil or self.step.requireExactName==false) and type(self.step.tracker)=="string" and string.find(targetname, self.step.tracker)) then
			self:Completed();
		elseif(type(self.step.tracker)=="table") then
			for _, v in ipairs(self.step.tracker) do
				if (self.step.requireExactName==true and string.find(targetname, v)) then
					self:Completed();
					break
				elseif ((self.step.requireExactName==nil or self.step.requireExactName==false) and string.find(targetname, v)) then
					self:Completed();
					break
				end
			end
		end
	end


	local player=Turbine.Gameplay.LocalPlayer.GetInstance();
	local target=player:GetTarget();
	self.playerName = player:GetName();
	if target~=nil then
		local targetname = target:GetName();	
		if(self.step.requireExactName==true and type(self.step.tracker)=="string" and targetname==self.step.tracker) then
			self:BuildChatTree(targetname);
			return;
		elseif((self.step.requireExactName==nil or self.step.requireExactName==false) and type(self.step.tracker)=="string" and string.find(targetname, self.step.tracker)) then
			self:BuildChatTree(targetname);
			return;
		elseif(type(self.step.tracker)=="table") then
			for _, v in ipairs(self.step.tracker) do
				if (self.step.requireExactName==true and string.find(targetname, v)) then
					self:BuildChatTree(targetname);
					return;
				elseif ((self.step.requireExactName==nil or self.step.requireExactName==false) and string.find(targetname, v)) then
					self:BuildChatTree(targetname);
					return;
				end
			end
		end
	end
	self.talkTree:SetVisible(false);
end

function StepReactorTALKTREE:LoadDialog(id)
	local dial = self.step.dials[id];
	self.currentDial = dial;
	if(dial==nil) then
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>YourOwnAdventures: ".._("TALK_UNK_REF").."</rgb>");
		return;
	end
	if(self.step.reverberate~=false) then
		Turbine.Shell.WriteLine("<rgb=#00FF00>\<"..self.targetName.."\></rgb> "..FormatAdventureString(dial.say));
	end
	self.talkTree:SetTargetDialog(dial.say);
	
	self.talkTree:ResetAnswers();
	
	function OnAnswerClick(answer)
		self:AnswerClick(answer);
	end;	
	
	for i, answer in pairs(dial.answers) do
		self.talkTree:AddAnswer(answer.say, answer, OnAnswerClick, answer.condition);
	end
	AdventureEngine:ApplyStepGenericActions(dial.onLoading);
end

function StepReactorTALKTREE:AnswerClick(answer)
	if(self.currentDial~=nil) then
		AdventureEngine:ApplyStepGenericActions(self.currentDial.onCompletion);
	end
	if(self.step.reverberateChat~=false) then
		Turbine.Shell.WriteLine("<rgb=#00FF00>\<"..self.playerName.."\></rgb> "..answer.say);
	end
	if(answer.nextDial~=nil) then
		AdventureEngine:ApplyStepGenericActions(answer);
		self:LoadDialog(answer.nextDial);
	elseif(answer.nextStep~=nil) then
		AdventureEngine:ApplyStepGenericActions(answer);
		self:Completed(answer);
	else
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>YourOwnAdventures: ERROR : last loaded answer has no .nextDial or .nextStep parameter</rgb>");
	end
end

function StepReactorTALKTREE:BuildChatTree(targetName)
	self.targetName = targetName;
	self.talkTree:SetTargetName(targetName);
	self:LoadDialog("START");
	self.talkTree:SetVisible();
	deedsEngine:AddDeed("GENERIC_DEED_TALK1");
end

function StepReactorTALKTREE:InitTargetLogger()
	if(Turbine.Gameplay.LocalPlayer.targetListerners == nil) then
		Turbine.Gameplay.LocalPlayer.targetListerners = {};
		local player=Turbine.Gameplay.LocalPlayer.GetInstance();
		player.TargetChanged = function (sender, args)
			for i, j in pairs(Turbine.Gameplay.LocalPlayer.targetListerners) do
				j:TargetChanged(sender, args);
			end
		end
	end
	if (Turbine.Gameplay.LocalPlayer.targetListerners ~= nil) then
		for i, j in pairs(Turbine.Gameplay.LocalPlayer.targetListerners) do
			if(j==self) then return; end
		end
		table.insert(Turbine.Gameplay.LocalPlayer.targetListerners, self);
	end	
end

function StepReactorTALKTREE:Completed(answer)
	if(AdventureEngine:TestSuccessCondition(self.step.successCondition)==false) then
		return;
	end
	self.step.nextStep = answer.nextStep;
	AdventureEngine:ApplyStepGenericActions(self.step.onCompletion);
	AdventureEngine:ClearSynchronousReactors(self);	
end

function StepReactorTALKTREE:Unload()
	self.talkTree:SetVisible(false);
	self.talkTree:ResetAnswers();		
	for i=1, table.getn(Turbine.Gameplay.LocalPlayer.targetListerners), 1 do
		if(Turbine.Gameplay.LocalPlayer.targetListerners[i]==self) then
			table.remove(Turbine.Gameplay.LocalPlayer.targetListerners, i);
			i=i-1;
		end
	end
end